@@ -7,15 +7,16 @@ https://lunchmoney.dev/#tags
 import logging
 from typing import List, Optional
 
-from pydantic import BaseModel, Field
+from pydantic import Field
 
 from lunchable._config import APIConfig
+from lunchable.models._base import LunchableModel
 from lunchable.models._core import LunchMoneyAPIClient
 
 logger = logging.getLogger(__name__)
 
 
-class TagsObject(BaseModel):
+class TagsObject(LunchableModel):
     """
     Lunchmoney Tags object
 
@@ -27,7 +28,7 @@ class TagsObject(BaseModel):
     description: Optional[str] = Field(description="User-defined description of tag")
 
 
-class _LunchMoneyTags(LunchMoneyAPIClient):
+class TagsClient(LunchMoneyAPIClient):
     """
     Lunch Money Tag Interactions
     """
@@ -45,7 +46,8 @@ class _LunchMoneyTags(LunchMoneyAPIClient):
         -------
         List[TagsObject]
         """
-        response_data = self._make_request(method=self.Methods.GET,
-                                           url_path=APIConfig.LUNCHMONEY_TAGS)
+        response_data = self._make_request(
+            method=self.Methods.GET, url_path=APIConfig.LUNCHMONEY_TAGS
+        )
         tag_objects = [TagsObject(**item) for item in response_data]
         return tag_objects
